==> date_ical-4.0.0.tar.gz: date_ical/src/Plugin/views/row/DateICal.php <==
<?php

namespace Drupal\date_ical\Plugin\views\row;

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\row\RowPluginBase;

/**
 * Row handler plugin for displaying Date iCal.
 *
 * @ingroup views_row_plugins
 *
 * @ViewsRow(
 *   id = "date_ical",
 *   title = @Translation("iCal Fields"),
 *   help = @Translation("Mapping iCal vs fields"),
 * )
 */
class DateICal extends RowPluginBase {

  /**
   * {@inheritdoc}
   */
  protected function defineOptions(): array {
    $options = parent::defineOptions();
    $options['date_field'] = ['default' => ''];
    $options['summary_field'] = ['default' => ''];
    $options['description_field'] = ['default' => ''];
    $options['location_field'] = ['default' => ''];
    $options['categories_field'] = ['default' => ''];
    $options['organizer_field'] = ['default' => ''];
    $options['status_field'] = ['default' => ''];
    $options['url_field'] = ['default' => ''];
    $options['attendee_field'] = ['default' => ''];
    $options['alarm_field'] = ['default' => ''];
    $options['attach_field'] = ['default' => ''];
    $options['geo_field'] = ['default' => ''];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function usesFields() {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state): void {
    parent::buildOptionsForm($form, $form_state);
    $listOption = $this->getFieldType();
    $textType = [
      'text',
      'text_long',
      'text_with_summary',
      'string',
      'string_long',
      'list_string',
      'unknown',
    ];
    $dateType = [
      'daterange',
      'datetime',
      'timestamp',
      'smartdate',
      'changed',
      'created',
    ];
    $form['date_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Date field'),
      '#options' => $this->getConfigurableFields($dateType, $listOption),
      '#default_value' => $this->options['date_field'],
      '#description' => $this->t('Please identify the field to use as the iCal date for each item in this view.
          Add a Date Filter or a Date Argument to the view to limit results to content in a specified date range.'),
      '#required' => TRUE,
    ];
    $form['summary_field'] = [
      '#type' => 'select',
      '#title' => $this->t('SUMMARY field'),
      '#empty_option' => $this->t('- None -'),
      '#options' => $this->getConfigurableFields($textType, $listOption),
      '#default_value' => $this->options['summary_field'],
      '#description' => $this->t('You may optionally change the SUMMARY component for each event in the iCal output.
        Choose which text, taxonomy term reference or Node Reference field you would like to be output as the SUMMARY.
        If using a Node Reference, the Title of the referenced node will be used.'),
    ];
    $form['description_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Description field'),
      '#description' => $this->t("The views field to use as the body text for each event (DESCRIPTION).<br>
          If you wish to include another field you can leave this field blank it will render the row."),
      '#options' => $this->getConfigurableFields($textType, $listOption),
      '#default_value' => $this->options['description_field'],
    ];
    $form['location_field'] = [
      '#type' => 'select',
      '#title' => $this->t('LOCATION field'),
      '#empty_option' => $this->t('- None -'),
      '#options' => $this->getConfigurableFields($textType + ['ad' => 'address'], $listOption),
      '#default_value' => $this->options['location_field'],
      '#description' => $this->t('You may optionally include a LOCATION component for each event in the iCal output.
        Choose which text or Node Reference field you would like to be output as the LOCATION.
        If using a Node Reference, the Title of the referenced node will be used.'),
    ];
    $form['geo_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Geo field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field to use as the geolocation for each event (GEO).'),
      '#options' => $this->getConfigurableFields([
        'geofield',
        'geolocation',
      ], $listOption),
      '#default_value' => $this->options['geo_field'],
    ];
    $form['categories_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Categories field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field to use as the categories for each event (CATEGORIES).'),
      '#options' => $this->getConfigurableFields([
        'entity_reference',
        'list_integer',
        'list_string',
      ], $listOption),
      '#default_value' => $this->options['categories_field'],
    ];
    $form['organizer_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Organizer field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field (email or user) to use as the Organizer for each event (ORGANIZER).'),
      '#options' => $this->getConfigurableFields(['entity_reference', 'email'], $listOption),
      '#default_value' => $this->options['organizer_field'],
    ];
    $form['attendee_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Participant field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field (email or user) to use as the attendee for each event (ATTENDEE).'),
      '#options' => $this->getConfigurableFields(['entity_reference', 'email'], $listOption),
      '#default_value' => $this->options['attendee_field'],
    ];
    $form['status_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Status field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field to use as the status for each event (STATUS), It must be empty or CONFIRMED,CANCELLED,TENTATIVE. Another value will crash'),
      '#options' => $this->getConfigurableFields(['list_string', 'list_states'], $listOption),
      '#default_value' => $this->options['status_field'],
    ];
    $form['url_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Url field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field to use as the URL for each event (URL).'),
      '#options' => $this->getConfigurableFields(['link', 'entity_reference'], $listOption),
      '#default_value' => $this->options['url_field'],
    ];
    $form['alarm_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Alarm field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) If it is an integer field, it means the number of minutes that will be reported.'),
      '#options' => $this->getConfigurableFields([
        'integer',
        'list_integer',
        'duration',
      ], $listOption),
      '#default_value' => $this->options['alarm_field'],
    ];
    $form['attach_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Attach field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field to use as the attach for each event (ATTACH).'),
      '#options' => $this->getConfigurableFields([
        'link',
        'file',
        'image',
        'entity_reference',
      ], $listOption),
      '#default_value' => $this->options['attach_field'],
    ];

  }

  /**
   * Get field type.
   */
  protected function getFieldType() {
    $listOption = [];
    $entityFieldManager = \Drupal::service('entity_field.manager');
    $entity_type = $this->view->getBaseEntityType()->id();
    $field_definitions = $entityFieldManager->getFieldStorageDefinitions($entity_type);
    $fields = $this->displayHandler->getHandlers('field');
    $labels = $this->displayHandler->getFieldLabels();
    foreach ($fields as $field_name => $field) {
      $field_definition = $field_definitions[$field_name] ?? $field->defineOptions();
      $field_type_links = 'unknown';
      if (is_array($field_definition)) {
        $field_type_links = $field_definition["type"]["default"] ?? 'unknown';
      }
      elseif (is_object($field_definition)) {
        $field_type_links = $field_definition->getType() ?? $field_definition["type"]["default"];
      }
      $listOption[$field_type_links][$field_name] = $labels[$field_name];
    }
    return $listOption;
  }

  /**
   * Get list of fields.
   */
  protected function getConfigurableFields($type = FALSE, $listFields = []) {
    $resultField = [];
    foreach ($listFields as $field_name => $fields) {
      if (in_array($field_name, $type)) {
        $resultField += $fields;
      }
    }
    return $resultField;
  }

  /**
   * Get event record.
   *
   * {@inheritDoc}
   */
  public function render($row) {
    $entity = $row->_entity ?? $row->_object->getValue();
    $style = $this->view->getStyle();
    $listOption = $this->getFieldType();
    $iCal = [];
    foreach ($this->options as $key => $fieldMapping) {
      if (!empty($fieldMapping) && is_string($fieldMapping)) {
        $iCal[$key] = $style->getFieldValue(0, $fieldMapping);
        if (empty($iCal[$key])) {
          continue;
        }
        if (!empty($entity->get('created'))) {
          $date = DrupalDateTime::createFromTimestamp($entity->get('created')->value, 'UTC');
          $iCal['created'] = $date->format("Y-m-d\TH:i:s");
        }
        if (!empty($entity->get('changed'))) {
          $date = DrupalDateTime::createFromTimestamp($entity->get('changed')->value, 'UTC');
          $iCal['last-modified'] = $date->format("Y-m-d\TH:i:s");
        }
        $type = '';
        if (isset($listOption["entity_reference"][$fieldMapping]) && !empty($entity->get($fieldMapping))) {
          $entityRef = $entity->get($fieldMapping)->referencedEntities();
          if (!empty($entityRef)) {
            $entityRef = current($entityRef);
            $type = $entityRef->getEntityTypeId();
          }
          $entityTitle = $entity->getTitle() ?? '';
          if (empty($entityTitle)) {
            foreach (['getDisplayName', 'getName', 'getLabel'] as $name) {
              if (method_exists($entity, $name)) {
                $entityTitle = $entity->$name();
                break;
              }
            }
          }
          $iCal[$key] = $entityTitle;
        }
        switch ($key) {
          case 'date_field':
          case 'geo_field':
            $iCal[$key] = current($entity->get($fieldMapping)->getValue());
            break;

          case 'location_field':
            $address = current($entity->get($fieldMapping)->getValue());
            if (is_array($address)) {
              $iCal[$key] = trim(implode(" ", array_reverse(array_filter($address))));
            }
            break;

          case 'organizer_field':
          case 'attendee_field':
            if ($type == 'user') {
              $iCal[$key] = [];
              foreach ($entity->get($fieldMapping)->referencedEntities() as $entityRef) {
                $iCal[$key][] = [
                  'name' => $entityRef->getDisplayName(),
                  'mail' => $entityRef->getEmail(),
                ];
              }
            }
            else {
              $invites = explode(', ', $entity->get($fieldMapping)
                ->getString());
              $iCal[$key] = [];
              foreach ($invites as $index => $invite) {
                preg_match_all('/([\w+\.]*\w+@[\w+\.]*\w+[\w+\-\w+]*\.\w+)/is', trim($invite), $matches);
                $mail = current($matches[0]);
                $iCal[$key][$index]['mail'] = $mail;
                $find = [$mail, '<', '>', '.'];
                $name = trim(ucwords(str_replace($find, ' ', $invite)));
                if (empty($name)) {
                  $ext = explode('@', $invite);
                  $name = trim(ucwords(str_replace($find, ' ', $ext[0])));
                }
                $iCal[$key][$index]['name'] = trim($name);
              }
            }
            break;
        }
      }
    }
    return $iCal;
  }

}

==> date_ical-4.0.1.tar.gz: date_ical/src/Plugin/views/row/DateICal.php <==
<?php

namespace Drupal\date_ical\Plugin\views\row;

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\views\Plugin\views\row\RowPluginBase;

/**
 * Row handler plugin for displaying Date iCal.
 *
 * @ingroup views_row_plugins
 *
 * @ViewsRow(
 *   id = "date_ical",
 *   title = @Translation("iCal Fields"),
 *   help = @Translation("Mapping iCal vs fields"),
 * )
 */
class DateICal extends RowPluginBase {

  /**
   * {@inheritdoc}
   */
  protected function defineOptions(): array {
    $options = parent::defineOptions();
    $options['date_field'] = ['default' => ''];
    $options['summary_field'] = ['default' => ''];
    $options['description_field'] = ['default' => ''];
    $options['location_field'] = ['default' => ''];
    $options['categories_field'] = ['default' => ''];
    $options['organizer_field'] = ['default' => ''];
    $options['status_field'] = ['default' => ''];
    $options['url_field'] = ['default' => ''];
    $options['attendee_field'] = ['default' => ''];
    $options['alarm_field'] = ['default' => ''];
    $options['attach_field'] = ['default' => ''];
    $options['geo_field'] = ['default' => ''];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function usesFields() {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state): void {
    parent::buildOptionsForm($form, $form_state);
    $listOption = $this->getFieldType();
    $textType = [
      'text',
      'text_long',
      'text_with_summary',
      'string',
      'string_long',
      'list_string',
      'unknown',
    ];
    $dateType = [
      'daterange',
      'datetime',
      'timestamp',
      'smartdate',
      'changed',
      'created',
    ];
    $form['date_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Date field'),
      '#options' => $this->getConfigurableFields($dateType, $listOption),
      '#default_value' => $this->options['date_field'],
      '#description' => $this->t('Please identify the field to use as the iCal date for each item in this view.
          Add a Date Filter or a Date Argument to the view to limit results to content in a specified date range.'),
      '#required' => TRUE,
    ];
    $form['summary_field'] = [
      '#type' => 'select',
      '#title' => $this->t('SUMMARY field'),
      '#empty_option' => $this->t('- None -'),
      '#options' => $this->getConfigurableFields($textType, $listOption),
      '#default_value' => $this->options['summary_field'],
      '#description' => $this->t('You may optionally change the SUMMARY component for each event in the iCal output.
        Choose which text, taxonomy term reference or Node Reference field you would like to be output as the SUMMARY.
        If using a Node Reference, the Title of the referenced node will be used.'),
    ];
    $form['description_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Description field'),
      '#description' => $this->t("The views field to use as the body text for each event (DESCRIPTION).<br>
          If you wish to include another field you can leave this field blank it will render the row."),
      '#options' => $this->getConfigurableFields($textType, $listOption),
      '#default_value' => $this->options['description_field'],
    ];
    $form['location_field'] = [
      '#type' => 'select',
      '#title' => $this->t('LOCATION field'),
      '#empty_option' => $this->t('- None -'),
      '#options' => $this->getConfigurableFields($textType + ['ad' => 'address'], $listOption),
      '#default_value' => $this->options['location_field'],
      '#description' => $this->t('You may optionally include a LOCATION component for each event in the iCal output.
        Choose which text or Node Reference field you would like to be output as the LOCATION.
        If using a Node Reference, the Title of the referenced node will be used.'),
    ];
    $form['geo_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Geo field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field to use as the geolocation for each event (GEO).'),
      '#options' => $this->getConfigurableFields([
        'geofield',
        'geolocation',
      ], $listOption),
      '#default_value' => $this->options['geo_field'],
    ];
    $form['categories_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Categories field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field to use as the categories for each event (CATEGORIES).'),
      '#options' => $this->getConfigurableFields([
        'entity_reference',
        'list_integer',
        'list_string',
      ], $listOption),
      '#default_value' => $this->options['categories_field'],
    ];
    $form['organizer_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Organizer field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field (email or user) to use as the Organizer for each event (ORGANIZER).'),
      '#options' => $this->getConfigurableFields(['entity_reference', 'email'], $listOption),
      '#default_value' => $this->options['organizer_field'],
    ];
    $form['attendee_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Participant field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field (email or user) to use as the attendee for each event (ATTENDEE).'),
      '#options' => $this->getConfigurableFields(['entity_reference', 'email'], $listOption),
      '#default_value' => $this->options['attendee_field'],
    ];
    $form['status_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Status field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field to use as the status for each event (STATUS), It must be empty or CONFIRMED,CANCELLED,TENTATIVE. Another value will crash'),
      '#options' => $this->getConfigurableFields(['list_string', 'list_states'], $listOption),
      '#default_value' => $this->options['status_field'],
    ];
    $form['url_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Url field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field to use as the URL for each event (URL).'),
      '#options' => $this->getConfigurableFields(['link', 'entity_reference'], $listOption),
      '#default_value' => $this->options['url_field'],
    ];
    $form['alarm_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Alarm field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) If it is an integer field, it means the number of minutes that will be reported.'),
      '#options' => $this->getConfigurableFields([
        'integer',
        'list_integer',
        'duration',
      ], $listOption),
      '#default_value' => $this->options['alarm_field'],
    ];
    $form['attach_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Attach field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field to use as the attach for each event (ATTACH).'),
      '#options' => $this->getConfigurableFields([
        'link',
        'file',
        'image',
        'entity_reference',
      ], $listOption),
      '#default_value' => $this->options['attach_field'],
    ];

  }

  /**
   * Get field type.
   */
  protected function getFieldType() {
    $listOption = [];
    $entityFieldManager = \Drupal::service('entity_field.manager');
    $entity_type = $this->view->getBaseEntityType()->id();
    $field_definitions = $entityFieldManager->getFieldStorageDefinitions($entity_type);
    $fields = $this->displayHandler->getHandlers('field');
    $labels = $this->displayHandler->getFieldLabels();
    foreach ($fields as $field_name => $field) {
      $field_definition = $field_definitions[$field_name] ?? $field->defineOptions();
      $field_type_links = 'unknown';
      if (is_array($field_definition)) {
        $field_type_links = $field_definition["type"]["default"] ?? 'unknown';
      }
      elseif (is_object($field_definition)) {
        $field_type_links = $field_definition->getType() ?? $field_definition["type"]["default"];
      }
      $listOption[$field_type_links][$field_name] = $labels[$field_name];
    }
    return $listOption;
  }

  /**
   * Get list of fields.
   */
  protected function getConfigurableFields($type = FALSE, $listFields = []) {
    $resultField = [];
    foreach ($listFields as $field_name => $fields) {
      if (in_array($field_name, $type)) {
        $resultField += $fields;
      }
    }
    return $resultField;
  }

  /**
   * Get event record.
   *
   * {@inheritDoc}
   */
  public function render($row) {
    $entity = $row->_entity ?? $row->_object->getValue();
    $style = $this->view->getStyle();
    $listOption = $this->getFieldType();
    $iCal = [];
    foreach ($this->options as $key => $fieldMapping) {
      if (!empty($fieldMapping) && is_string($fieldMapping)) {
        $iCal[$key] = $style->getFieldValue($row->index, $fieldMapping);
        if (is_array($iCal[$key]) && !empty($iCal[$key]['value'])) {
          $iCal[$key] = $iCal[$key]['value'];
        }
        if (empty($iCal[$key])) {
          continue;
        }
        if ($entity->hasField('created') && !empty($entity->get('created'))) {
          $date = DrupalDateTime::createFromTimestamp($entity->get('created')->value, 'UTC');
          $iCal['created'] = $date->format("Y-m-d\TH:i:s");
        }
        if ($entity->hasField('changed') && !empty($entity->get('changed'))) {
          $date = DrupalDateTime::createFromTimestamp($entity->get('changed')->value, 'UTC');
          $iCal['last-modified'] = $date->format("Y-m-d\TH:i:s");
        }
        $type = '';
        if (isset($listOption["entity_reference"][$fieldMapping]) && !empty($entity->get($fieldMapping))) {
          $entityRef = $entity->get($fieldMapping)->referencedEntities();
          if (!empty($entityRef)) {
            $entityRef = current($entityRef);
            $type = $entityRef->getEntityTypeId();
          }
          foreach (['getTitle', 'getDisplayName', 'getName', 'getLabel'] as $name) {
            if (method_exists($entity, $name)) {
              $entityTitle = $entity->$name();
              break;
            }
          }
          $iCal[$key] = $entityTitle ?? '';
        }
        switch ($key) {
          case 'date_field':
          case 'geo_field':
            $iCal[$key] = current($entity->get($fieldMapping)->getValue());
            break;

          case 'location_field':
            $address = current($entity->get($fieldMapping)->getValue());
            if (is_array($address)) {
              $iCal[$key] = trim(implode(" ", array_reverse(array_filter($address))));
            }
            break;

          case 'organizer_field':
          case 'attendee_field':
            if ($type == 'user') {
              $iCal[$key] = [];
              foreach ($entity->get($fieldMapping)->referencedEntities() as $entityRef) {
                $iCal[$key][] = [
                  'name' => $entityRef->getDisplayName(),
                  'mail' => $entityRef->getEmail(),
                ];
              }
            }
            else {
              $invites = explode(', ', $entity->get($fieldMapping)
                ->getString());
              $iCal[$key] = [];
              foreach ($invites as $index => $invite) {
                preg_match_all('/([\w+\.]*\w+@[\w+\.]*\w+[\w+\-\w+]*\.\w+)/is', trim($invite), $matches);
                $mail = current($matches[0]);
                $iCal[$key][$index]['mail'] = $mail;
                $find = [$mail, '<', '>', '.'];
                $name = trim(ucwords(str_replace($find, ' ', $invite)));
                if (empty($name)) {
                  $ext = explode('@', $invite);
                  $name = trim(ucwords(str_replace($find, ' ', $ext[0])));
                }
                $iCal[$key][$index]['name'] = trim($name);
              }
            }
            break;

          case 'summary_field':
          case 'description_field':
            // Remove HTML.
            if (!empty($iCal[$key]) && is_string($iCal[$key])) {
              $iCal[$key] = preg_replace('/<br(\s+)?\/?>/i', "\n", $iCal[$key]);
              $iCal[$key] = trim(strip_tags($iCal[$key], '<a>'));
            }
            break;

          case 'url_field':
            if (!empty($iCal[$key])) {
              $parsedUrl = parse_url($iCal[$key]);
              if ($parsedUrl['scheme'] == 'internal') {
                $iCal[$key] = Url::fromUri($iCal[$key], ['absolute' => TRUE])->toString();
                $parsedUrl = parse_url($iCal[$key]);
              }
              if (!in_array($parsedUrl['scheme'], ['https', 'http'])) {
                $iCal[$key] = FALSE;
              }
            }
            break;
        }
      }
    }
    return $iCal;
  }

}
